import requests
import json
import os
import urllib

from cis_client.lib.aaa.token import Token


class AuthClient(object):

    _token_cls = Token
    restapi_version = "v2"
    token_endpoint = "tokens"
    auth_header = "X-Auth-Token"

    @staticmethod
    def _compose_endpoint_url(token_endpoint_url, token, params):
        endpoint_url = os.path.join(token_endpoint_url, token)
        if params:
            query_string = urllib.urlencode(params)
            endpoint_url = ''.join([endpoint_url, '?', query_string])
        return endpoint_url

    def validate_token(self, token, user_token=None, params=None):
        endpoint_url = self._compose_endpoint_url(
            self._auth_server.token_endpoint_url, token, params)
        token_response = requests.get(
            endpoint_url,
            verify=(not self._insecure),
            headers = self.get_auth_header(user_token or token))
        token_response.raise_for_status()

        try:
            token_ctx = json.loads(token_response.content)
            token_obj = self._token_cls(token, token_ctx, service_name=self._service_name)
        except ValueError:
            raise
        except Exception as token_key_ctx_err:
            raise Exception("Invalid token context: {}".format(token_key_ctx_err))

        return token_obj

    def __init__(self, aaa_hostname, service_name=None, insecure=False):
        self._service_name = service_name
        self._token_endpoint_url = "{}/{}/{}".format(
            aaa_hostname, self.restapi_version, self.token_endpoint)
        self._insecure = insecure

    def get_auth_header(self, token):
        return {self.auth_header: token}

    def get_token_with_context(self, token, context):
        post_token_response = requests.post(self._token_endpoint_url,
                                            headers=self.get_auth_header(token),
                                            verify=(not self._insecure),
                                            json={"token": token,
                                                  Token.ClientCtxAttrName: context})
        post_token_response.raise_for_status()
        return json.loads(post_token_response.content)

    def get_token(self, username, password):
        post_token_response = requests.post(self._token_endpoint_url,
                                            verify=(not self._insecure),
                                            json={"username": username, "password": password})
        post_token_response.raise_for_status()
        return json.loads(post_token_response.content)

    def delete_token(self, token, user_token):
        del_token_response = requests.delete(os.path.join(self._auth_server.token_endpoint_url, token),
                                             headers=self.get_auth_header(user_token),
                                             verify=(not self._insecure))
        del_token_response.raise_for_status()



import os
import sys
import pkg_resources
import click
click.disable_unicode_literals_warning = True

from cis_client.commands import utils


app_version = pkg_resources.working_set.by_key['udn-cis-client']


class Context(object):

    def __init__(self):
        self.verbose = False

    def log(self, msg, *args):
        """Logs a message to stderr."""
        if args:
            msg %= args
        click.echo(msg, file=sys.stderr)

    def vlog(self, msg, *args):
        """Logs a message to stderr only if verbose is enabled."""
        if self.verbose:
            self.log(msg, *args)


pass_context = click.make_pass_decorator(Context, ensure=True)
cmd_folder = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                          'commands'))


class CisClientCLI(click.MultiCommand):
    def load_commands(self):
        from cis_client.commands import cmd_job_status
        from cis_client.commands import cmd_upload
        self.commands = {
            'upload': cmd_upload.cli,
            'job_status': cmd_job_status.cli
        }

    def list_commands(self, ctx):
        self.load_commands()
        rv = self.commands.keys()
        rv.sort()
        return rv

    def get_command(self, ctx, name):
        self.load_commands()
        return self.commands.get(name)


@click.command(cls=CisClientCLI, context_settings=utils.CONTEXT_SETTINGS)
@click.option('-v', '--verbose', is_flag=True,
              help='Enables verbose mode.')
@click.version_option(version=app_version.version)
@pass_context
def cli(ctx, verbose):
    ctx.verbose = verbose



import requests
import os.path

from cis_client.lib.cis_north import access_key_client
from cis_client.lib import chunk_upload


class ContentClient(object):
    api_version = 'v1'

    def __init__(self, hostname, port, schema='http', insecure=False):
        super(ContentClient, self).__init__()
        self.schema = schema
        self.hostname = hostname
        self.port = port
        self.insecure = insecure

    def get_endpoint(self, dest_path):
        endpoint = "{schema}://{hostname}:{port}/{api_version}/{dest_path}".format(
            schema=self.schema,
            hostname=self.hostname,
            port=self.port,
            api_version=self.api_version,
            dest_path=dest_path)
        return endpoint

    def upload(self, access_key, path, dest_path, progress_callback=None):
        uploader = chunk_upload.UploadChunks(path, progress_callback=progress_callback)
        response = requests.post(
            self.get_endpoint(dest_path),
            verify=(not self.insecure),
            headers={"X-Auth-Token": access_key},
            data=chunk_upload.IterableToFileAdapter(uploader))
        response.raise_for_status()
        return response


def http_upload(aaa_host, username, password, north_host,
                ingest_point, path, destination_path=None,
                progress_callback=None, **kwargs):
    """HTTP upload"""

    if destination_path is None:
        destination_path = os.path.basename(path)

    ingest_point_info, access_key = access_key_client.get_access_key(
        aaa_host, username, password, north_host, ingest_point, **kwargs)

    gateway_hostname = ingest_point_info['gateway']['hostname']
    gateway_port = ingest_point_info['gateway']['protocols']['http']['port']
    content_client = ContentClient(gateway_hostname, gateway_port, insecure=kwargs['insecure'])
    return content_client.upload(access_key, path, destination_path, progress_callback)



import json
import requests
import urllib3

from cis_client.lib.aaa.auth_client import AuthClient

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class JobsClient(object):
    jobs_endpoint = 'jobs'

    def __init__(self, north_host, token, insecure=False):
        super(JobsClient, self).__init__()
        self.jobs_endpoint_url = "{}/{}".format(north_host, self.jobs_endpoint)
        self.token = token
        self.insecure = insecure

    def get_auth_header(self):
        return {AuthClient.auth_header: self.token}

    def list(self, **kwargs):
        query_jobs_url = self.compose_query_jobs_url(**kwargs)
        get_jobs_response = requests.get(
            query_jobs_url,
            verify=(not self.insecure),
            headers=self.get_auth_header())
        get_jobs_response.raise_for_status()
        return json.loads(get_jobs_response.content)

    def compose_query_jobs_url(self, **kwargs):
        join_values = lambda list_values: ','.join(list_values)
        query_params = []
        if kwargs.get('job_type') is not None:
            query_params.append('type=%s' % join_values(kwargs['job_type']))
        if kwargs.get('path') is not None:
            query_params.append('filename=%s' % join_values(kwargs['path']))
        if kwargs.get('file_basename') is not None:
            query_params.append(
                'filter_by=file_basename&filter_value=%s' % kwargs['file_basename'])
        if kwargs.get('case_sensitive_file_basename') is not None:
            query_params.append('filter_case_sensitive=%s' % 'true' if kwargs['filter_case_sensitive'] else 'false')
        if kwargs.get('state') is not None:
            query_params.append('state=%s' % join_values(kwargs['state']))
        if kwargs.get('state_simple') is not None:
            query_params.append('state_simple=%s' % join_values(kwargs['state_simple']))
        if kwargs.get('fields') is not None:
            query_params.append('fields=%s' % join_values(kwargs['fields']))
        if kwargs.get('with_children') is not None:
            query_params.append('with_children=%s' % 'true' if kwargs['with_children'] else 'false')
        if kwargs.get('ingest_point') is not None:
            query_params.append('ingest_point=%s' % kwargs['ingest_point'])
        if kwargs.get('root_only') is not None:
            query_params.append('root_only=%s' % 'true' if kwargs['root_only'] else 'false')
        if kwargs.get('sort_by') is not None:
            query_params.append('sort_by=%s' % kwargs['sort_by'])
        if kwargs.get('sort_order') is not None:
            query_params.append('sort_order=%s' % kwargs['sort_order'])
        if kwargs.get('offset') is not None:
            query_params.append('offset=%s' % str(kwargs['offset']))
        if kwargs.get('page_size') is not None:
            query_params.append('page_size=%s' % str(kwargs['page_size']))
        if kwargs.get('brand_id') is not None:
            query_params.append('brand_id=%s' % str(kwargs['brand_id']))
        if kwargs.get('account_id') is not None:
            query_params.append('account_id=%s' % str(kwargs['account_id']))
        if kwargs.get('group_id') is not None:
            query_params.append('group_id=%s' % str(kwargs['group_id']))
        url = '{jobs_endpoint_url}?{params}'.format(
            jobs_endpoint_url=self.jobs_endpoint_url,
            params='&'.join(query_params)
        )
        return url


def get_jobs(aaa_host, username, password, north_host, **kwargs):
    """Gets jobs

    :param aaa_host:
    :param username:
    :param password:
    :param north_host:
    :param kwargs: can contains
        - brand_id: Brand ID
        - account_id: account ID
        - group_id: group ID
        - job_type: list of job types like ['abr'] or ['http-push'] or ['abr', 'http-push'] or other
        - path: list of exact paths like ['root_dir/sub_dir/vid.mp4']
        - file_basename: partial match of filename.
            Filtering by filename allows to specify the beginning of filename only.
        - case_sensitive_file_basename: True if filtering is case sensitive.
        - state: list of allowed states. Allowed states are:
            pending
            running
            cancelling
            paused
            completed
            failed
        - fields: list of fields in response like ['id', 'filename'].
        - with_children: if True will return whole job tree which contains information
            about children and children of children
        - ingest_point: ingest point id
        - root_only: returns root jobs only
        - sort_by: field name to sort by
        - sort_order: can be 'asc' or 'desc'
        - offset: offset
        - page_size: size of page, -1 means all rows
        - insecure: True if SSL verification must be skipped

    :return: list of jobs
    """
    auth_client = AuthClient(aaa_host, insecure=kwargs['insecure'])
    token = auth_client.get_token(username, password)
    jobs_client = JobsClient(north_host, token, insecure=kwargs['insecure'])
    return jobs_client.list(**kwargs)



import os
import os.path
import subprocess
import shlex

from cis_client.lib.cis_north import access_key_client


def aspera_upload(
        aaa_host, username, password,
        north_host, ingest_point, path, destination_path=None, **kwargs):
    """Aspera upload"""

    if destination_path is None:
        destination_path = os.path.basename(path)
    destination_path = destination_path.lstrip('/')

    ingest_point_info, access_key = access_key_client.get_access_key(
        aaa_host, username, password, north_host, ingest_point, **kwargs)

    # 'ASPERA_SCP_PASS=<access-key> ascp -P 33001 --user=<UDN-username>
    #   --host=b.aspera.cdx-dev.dataingest.net --mode=send <local-path> <dest-path>'
    aspera_template = ingest_point_info['gateway']['protocols']['aspera']['template']
    aspera_cmd = aspera_template.replace('<dest-path>', destination_path, 1).\
                                 replace('<local-path>', path, 1). \
                                 replace('<UDN-username>', username, 1). \
                                 replace('<access-key>', access_key, 1)
    aspera_cmd_args = shlex.split(aspera_cmd)
    aspera_cmd_args_resume = [aspera_cmd_args[1], '-k2', '-l1G']
    aspera_cmd_args_resume.extend(aspera_cmd_args[2:])
    subprocess.call(aspera_cmd_args_resume, env=dict(os.environ, **{'ASPERA_SCP_PASS': access_key}))



import json
import requests
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def get_cis_version(north_host, **kwargs):
    """Gets CIS version

    :param north_host:
    :param kwargs: can contains
        - insecure: True if SSL verification must be skipped
    :return: string CIS version
    """

    version_endpoint_url = "{}/version".format(north_host)
    response = requests.get(
        version_endpoint_url,
        verify=True if not kwargs.get('insecure') else False)
    response.raise_for_status()
    return json.loads(response.content)



class Token(object):

    BrandAttrName = "brand_id"
    AccountAttrName = "account_id"
    GroupAttrName = "group_id"
    UsernameAttrName = "username"
    ClientCtxAttrName = "client_context"
    ServicesAttrName = "services"
    PermissionsAttrName = "permissions"
    ResourcesAttrName = "resources"

    SuperAdminFlag = "is_super_admin"
    BrandUserFlag = "is_brand_user"
    AccountUserFlag = "is_account_user"
    GroupUserFlag = "is_group_user"

    AuthIdAttrs = [BrandAttrName, AccountAttrName, GroupAttrName] # NOTE: make sure the list is sorted as now. DON'T REORDER ITEMS

    Header = "X-Auth-Token"
    ServiceName = "CIS"

    @property
    def key(self):
        return self._token_key

    @property
    def brand(self):
        return self.brand_id

    @property
    def brand_id(self):
        return str(self._token_ctx[self.BrandAttrName])

    @property
    def account(self):
        return self.account_id

    @property
    def account_id(self):
        return str(self._token_ctx[self.AccountAttrName])

    @property
    def group(self):
        return self.group_id

    @property
    def group_id(self):
        # CIS has a policy that the first group in a list is used
        group_id = self._token_ctx[self.GroupAttrName]
        return str(group_id[0] if isinstance(group_id, list) and len(group_id) != 0 else group_id)

    @property
    def username(self):
        return self._token_ctx[self.UsernameAttrName]

    @property
    def is_super_admin(self):
        return self._token_ctx[self.SuperAdminFlag]

    @property
    def is_brand_user(self):
        return self._token_ctx[self.BrandUserFlag]

    @property
    def is_account_user(self):
        return self._token_ctx[self.AccountUserFlag]

    @property
    def is_group_user(self):
        return self._token_ctx[self.GroupUserFlag]

    @property
    def client_context(self):
        return self._token_ctx.get(self.ClientCtxAttrName, None)

    @property
    def permissions(self):
        return self._token_ctx[self.ServicesAttrName][self.ServiceName][self.PermissionsAttrName][self.ResourcesAttrName]

    @property
    def json(self):
        return dict(self._token_ctx)

    @property
    def header(self):
        return {self.Header: self.key}

    @staticmethod
    def auth_header(token):
        return {Token.Header: token}

    @property
    def level(self):
        try:
            return self._level
        except AttributeError: # just performance improvement
            # It's important to keep this switch in the current order, since AAA is so consistent that it can return both account_user + super_admin flags, as well as more then 1 group)))
            if self.is_group_user:
                self._level = 3
            elif self.is_account_user:
                self._level = 2
            elif self.is_brand_user:
                self._level = 1
            elif self.is_super_admin:
                self._level = 0
            else:
                raise KeyError("Inconsistent token has been discovered {}".format(token))
            return self._level

    @property
    def required_auth_ids(self):
        return self.AuthIdAttrs[:self.level]

    def __contains__(self, key):
        return bool(self._token_ctx.get(key))

    def __getitem__(self, key):
        try:
            return getattr(self, key)
        except AttributeError:
            raise KeyError(key)

    def __eq__(self, other):
        if not isinstance(other, Token):
            raise TypeError("A right operand must be Token type: {}".format(other))

        return self.key == other.key and self.brand == other.brand and self.account == other.account and \
                                         self.group == other.group and self.username == other.username

    def __init__(self, token_key, token_ctx, service_name=None):

        # NOTE: that CIS North and CIS Storage/Gateways are represented in AAA as a single 'CIS' service
        # defined by Token.ServiceName, but 'CIS-south' is a different Service in AAA.
        # That's the first use-case why ServiceName becomes parametrizeable
        if service_name: # override what is defined in the class by default
            self.ServiceName = service_name

        if not isinstance(token_key, (str, unicode)):
            raise TypeError("Token key must be a string type !!!")
        if not isinstance(token_ctx, dict):
            raise TypeError("Token context must be an object of dict class !!!")

        if self.ServiceName not in token_ctx[self.ServicesAttrName] or \
                self.PermissionsAttrName not in token_ctx[self.ServicesAttrName][self.ServiceName]:
            raise ValueError("Missing permissions in the given token context {}".format(token_ctx))

        self._token_key = token_key
        self._token_ctx = token_ctx




class BaseClient(object):
    def get_auth_params(self, **kwargs):
        auth_params = {
            key: value
            for key, value in kwargs.items()
            if key in ('brand_id', 'account_id', 'group_id')
        }
        return auth_params



import terminaltables
from textwrap import wrap
import os


def get_terminal_size():
    try:
        rows, columns = os.popen('stty size', 'r').read().split()
    except Exception as e:
        # FIXME tmp decission for windows or other...
        return (60, 120)
    return (int(rows), int(columns))


def print_json_as_table(data, header_field_map=None, order_fields=None, wrap_text=True):
    rows, cols = get_terminal_size()
    keys = set()
    for row in data:
        row_keys = row.keys()
        keys.update(row_keys)
    ordered_keys = order_fields or keys
    cols_len = len(ordered_keys)
    col_width = cols/cols_len
    rows_data = []
    for row in data:
        row_data = []
        for key in ordered_keys:
            data = unicode(row.get(key))
            # do not wrap 'id' column because we usually must copy it from terminal
            if key == 'id':
                wrapped_string = data
            else:
                wrapped_string = '\n'.join(wrap(data, col_width, replace_whitespace=False)) if wrap_text else data
            row_data.append(wrapped_string)
        rows_data.append(row_data)

    # replace table fields with user friendly fields in header
    if header_field_map:
        ordered_keys = map(lambda field_name: header_field_map.get(field_name, field_name), ordered_keys)

    table_data = [list(ordered_keys)]
    table_data.extend(rows_data)
    table = terminaltables.AsciiTable(table_data)

    table.inner_row_border = True
    print(table.table.encode('utf-8'))



from __future__ import unicode_literals

from functools import wraps
import sys
import imp

import requests
import click

import cis_client
from cis_client import exception
from cis_client.lib.cis_north import version_client

# import progressbar2 here (not progressbar)
correct_progressbar_loaded = False
for path in sys.path:
    try:
        imp.load_module('progressbar2', *imp.find_module('progressbar', [path]))
        import progressbar2
        pb = progressbar2.ProgressBar(max_value=100)
        correct_progressbar_loaded = True
        break
    except (ImportError, TypeError) as e:
        pass
if correct_progressbar_loaded is False:
    print("Please reinstall application. progressbar2 is not installed")
    sys.exit(1)


CONTEXT_SETTINGS = dict(auto_envvar_prefix='CIS_CLIENT')


def add_auth_options(func):
    @click.option('--brand-id', type=click.STRING, help='Brand Id')
    @click.option('--account-id', type=click.STRING, help='Account Id')
    @click.option('--group-id', type=click.STRING, help='Group Id')
    @wraps(func)
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)
    return wrapper


def add_host_options(func):
    @click.option('--aaa-host', required=True, type=click.STRING, help='AAA hostname.')
    @click.option('--north-host', required=True, type=click.STRING, help='CIS North hostname.')
    @click.option('--insecure', type=click.BOOL, default=False,
                  help='Allow insecure server connections when using SSL')
    @wraps(func)
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)
    return wrapper


def add_credentials_options(func):
    @click.option('--username', required=True, type=click.STRING, help='AAA username.')
    @click.option('--password', required=True, type=click.STRING, help='AAA password.')
    @wraps(func)
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)
    return wrapper


def handle_exceptions(func):
    def wrapper(context, *args, **kwargs):
        try:
            func(context, *args, **kwargs)
        except requests.HTTPError as e:
            print("Error: {}, reason: {}".format(str(e), e.response.text).encode('utf-8'))
            sys.exit(1)
        except (exception.OptionException, exception.UploadConflictException) as e:
            print("Error: {}".format(e.message).encode('utf-8'))
            sys.exit(1)
        except Exception as e:
            if context.verbose is True:
                raise
            else:
                print("Error: {}".format(str(e.message)).encode('utf-8'))
    return wrapper


def get_source_file_list_from_kwargs(**kwargs):
    split_values = lambda comma_separated_values: map(str.strip, map(str, comma_separated_values.split(',')))
    source_file_list = kwargs.get('source_file_list')
    source_file = kwargs.get('source_file')
    if source_file_list is not None and source_file is not None:
        raise exception.OptionException("Please specify only one option --source-file-list or --source-file")
    if not source_file_list and not source_file:
        raise exception.OptionException("Please specify either option --source-file-list or --source-file option")
    paths = []
    if source_file_list is not None:
        with open(source_file_list) as f:
            file_content = f.read()
        paths = map(str.strip, file_content.strip().split('\n'))
    if source_file is not None:
        paths = split_values(source_file)

    return paths


def check_cis_version(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        cis_version = version_client.get_cis_version(**kwargs)
        if isinstance(cis_version, basestring):
            if not cis_version.startswith(cis_client.SUPPORTED_CIS_VERSION):
                print("Backend CIS version {} is not supported by udn-cis-client. Please install a new version of udn-cis-client:\n"\
                      "pip install --upgrade udn-cis-client".format(cis_version))
                sys.exit(1)
        else:  # dict
            from cis_client import cli
            cli_version_tuple = cli.app_version.version.split('.')
            cli_version_major = str(cli_version_tuple[0])
            cli_version_minor = str(cli_version_tuple[1])
            cis_version_tuple = cis_version['cis-client'].split('.')
            cis_version_major = str(cis_version_tuple[0])
            cis_version_minor = str(cis_version_tuple[1])
            if cli_version_major != cis_version_major or cli_version_minor != cis_version_minor:
                print("Backend CIS version is not supported by udn-cis-client. Current version of udn-cis-client is {cli_version}.\n"\
                      "To connect to server please install a new version of udn-cis-client:\n"\
                      "pip install udn-cis-client=={cis_version_major}.{cis_version_minor}".format(
                    cli_version=cli.app_version,
                    cis_version_major=cis_version_major,
                    cis_version_minor=cis_version_minor).encode('utf-8'))
                sys.exit(1)
        func(*args, **kwargs)
    return wrapper


class ProgressBar(progressbar2.ProgressBar):
    def __init__(self, *args, **kwargs):
        super(ProgressBar, self).__init__(*args, **kwargs)
        self.exception = None

    def __exit__(self, exc_type, exc_value, traceback):
        if exc_type is not None:
            self.exception = exc_type
        self.finish()

    def update(self, value=None, force=False, **kwargs):
        if self.exception:
            value = 0
        super(ProgressBar, self).update(value, force, **kwargs)



import json
import requests

import urllib3

from cis_client.lib.cis_north import base_client


urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class IngestPointClient(base_client.BaseClient):
    def __init__(self, north_host, insecure=False):
        super(IngestPointClient, self)
        self.north_host = north_host
        self.insecure = insecure

    def get_ingest_point(self, auth_token, ingest_point, **kwargs):
        endpoint = self.get_endpoint(ingest_point)
        response = requests.get(
            endpoint,
            verify=(not self.insecure),
            headers={"X-Auth-Token": auth_token},
            params=self.get_auth_params(**kwargs)
        )
        response.raise_for_status()
        return json.loads(response.content)

    def get_endpoint(self, ingest_point):
        endpoint = '{north_host}/ingest_points/{ingest_point}'.format(
            north_host=self.north_host,
            ingest_point=ingest_point
        )
        return endpoint



class OptionException(Exception):
    pass


class UploadConflictException(Exception):
    def __init__(self, destination_path):
        self.message = "Conflict. File {} exists on remote server.".format(destination_path)



import errno
import os.path
import paramiko

from cis_client.lib.cis_north import access_key_client
from cis_client import exception


def sftp_file_exist(sftp_client, destination_path):
    try:
        sftp_client.stat(destination_path)
    except IOError as e:
        if e.errno == errno.ENOENT:
            return False
        raise
    return True


def sftp_upload(aaa_host, username, password, north_host,
                ingest_point, path, destination_path=None,
                progress_callback=None, **kwargs):
    """SFTP upload"""

    if destination_path is None:
        destination_path = os.path.basename(path)

    ingest_point_info, access_key = access_key_client.get_access_key(
        aaa_host, username, password, north_host, ingest_point, **kwargs)

    gateway_hostname = ingest_point_info['gateway']['hostname']
    gateway_port = ingest_point_info['gateway']['protocols']['sftp']['port']

    with paramiko.Transport((gateway_hostname, gateway_port)) as transport:
        transport.connect(username=username, password=access_key)
        with paramiko.SFTPClient.from_transport(transport) as sftp_client:
            if sftp_file_exist(sftp_client, destination_path):
                raise exception.UploadConflictException(destination_path)
            sftp_client.put(path, destination_path, callback=progress_callback)



import click

from cis_client import cli
from cis_client.commands import utils
from cis_client.lib.cis_north import jobs_client
from cis_client import exception
from cis_client.commands import printer


@click.command('job_status', context_settings=utils.CONTEXT_SETTINGS, help='Gets status of jobs.')
@click.option('--ingest-point', type=click.STRING,
              help='Ingest point')
@click.option('--path-list-file', type=click.Path(resolve_path=True),
              help='Path to file that contains list of full paths to content separated by new line symbol.')
@click.option('--path-list', type=click.STRING,
              help='Comma separated list of full paths to content.')
@click.option('--job-type', type=click.STRING,
              help='Job type like "abr" or "http-push" or "abr,http-push" or other')
@click.option('--state', type=click.STRING,
              help='an integer representation of composited states: pending, ready, scheduling, starting, '
                   'running, suspending, pausing, resuming, cancelling, completed, suspended, paused, failed, '
                   'cancelled, done. General purpose - for debugging.')
@click.option('--state-simple', type=click.STRING,
              help='Human readable status. Can contain values: "pending", "running", "cancelling", '
                   '"paused", "completed", "failed" in job query response.')
@click.option('--fields', type=click.STRING, default='id,filename,progress,state,name',
              help='Comma separated list of fields that will be in response')
@click.option('--root-only', type=click.BOOL, default=True,
              help='Boolean. Shows root only or all jobs.')
@click.option('--page-size', type=click.INT, default=1000,
              help='Max size of data that will be returned in response.')
@click.option('--offset', type=click.INT,
              help='Begin result set at this index.')
@click.option('--wrap-table', default=True, type=click.BOOL,
              help='Wrap result table to fit with screen.')
@utils.add_host_options
@utils.add_auth_options
@utils.add_credentials_options
@cli.pass_context
@utils.handle_exceptions
@utils.check_cis_version
def cli(ctx, **kwargs):
    split_values = lambda comma_separated_values: map(str.strip, map(str, comma_separated_values.split(',')))
    if kwargs.get('job_type') is not None:
        kwargs['job_type'] = split_values(kwargs['job_type'])
    if kwargs.get('path_list_file') is not None and kwargs.get('path_list') is not None:
        raise exception.OptionException("Please specify only one option --path-list-file or --path-list")
    if kwargs.get('path_list_file') is not None:
        with open(kwargs['path_list_file']) as f:
            file_content = f.read()
        path = map(str.strip, file_content.strip().split('\n'))
        kwargs['path'] = path
    if kwargs.get('path_list') is not None:
        kwargs['path'] = split_values(kwargs['path_list'])
    if kwargs.get('state') is not None:
        kwargs['state'] = split_values(kwargs['state'])
    if kwargs.get('state_simple') is not None:
        kwargs['state_simple'] = split_values(kwargs['state_simple'])
    if kwargs.get('fields') is not None:
        kwargs['fields'] = split_values(kwargs['fields'])
    kwargs['with_children'] = True
    jobs = jobs_client.get_jobs(
        kwargs.pop('aaa_host'), kwargs.pop('username'), kwargs.pop('password'), kwargs.pop('north_host'), **kwargs)
    for job_data in jobs['data']:
        if 'progress' in job_data:
            try:
                job_data['progress'] = float(job_data['progress']) * 100
            except (ValueError, TypeError):
                job_data['progress'] = ''
        if 'children_jobs' in job_data and kwargs.get('path'):
            job_data['subjobs progress'] = ''
            if job_data.get('state_simple') == 'completed':
                # do not show subjobs progress for completed tasks
                continue
            for subjob in job_data['children_jobs']:
                job_data['subjobs progress'] += '"{}" has status "{}"\n'.format(
                    subjob.get('name'), subjob.get('simple_state') or subjob.get('state'))
    table_fields = kwargs.get('fields')
    table_fields = filter(lambda field: field != 'name', table_fields)  # remove name of root job
    if table_fields and kwargs.get('path'):
        table_fields.append('subjobs progress')
    if ('state' in table_fields and
            jobs['data'] and jobs['data'][0].get('state_simple')):  # try to do it independently from CIS version
        table_fields = map(lambda field: 'state_simple' if field == 'state' else field, table_fields)
    printer.print_json_as_table(jobs['data'], header_field_map={'state_simple': 'state'},
                                order_fields=table_fields, wrap_text=kwargs['wrap_table'])



import os


class UploadChunks(object):
    def __init__(self, filename, chunk_size=1 << 15, progress_callback=None):
        self.filename = filename
        self.chunk_size = chunk_size
        self.progress_callback = progress_callback
        self.total_size = os.path.getsize(filename)
        self.readsofar = 0

    def __iter__(self):
        with open(self.filename, 'rb') as file:
            if self.progress_callback:
                self.progress_callback(0)
            while True:
                data = file.read(self.chunk_size)
                if not data:
                    if self.progress_callback:
                        self.progress_callback(self.total_size)
                    break
                self.readsofar += len(data)
                if self.progress_callback:
                    self.progress_callback(self.readsofar)
                yield data

    def __len__(self):
        return self.total_size


class IterableToFileAdapter(object):
    def __init__(self, iterable):
        self.iterator = iter(iterable)
        self.length = len(iterable)

    def read(self, size=-1):
        return next(self.iterator, b'')

    def __len__(self):
        return self.length



from setuptools import setup

setup(
    name='udn-cis-client',
    version='1.3.4',
    description='UDN CIS client tool',
    packages=[
        'cis_client',
        'cis_client.commands',
        'cis_client.lib',
        'cis_client.lib.aaa',
        'cis_client.lib.cis_north',
        'cis_client.lib.cis_gateway',
    ],
    include_package_data=True,
    install_requires=[
        'click',
        'terminaltables',
        'requests',
        'progressbar2',
        'paramiko'
    ],
    entry_points='''
        [console_scripts]
        udn-cis-client=cis_client.cli:cli
    ''',
    zip_safe=False,
    script_name='setup.py',
    author='Marian Horban',
    author_email='mhorban@vidscale.com'
)



import json
import requests
import urllib3

from cis_client.lib.cis_north import base_client
from cis_client.lib.aaa.auth_client import AuthClient
from cis_client.lib.cis_north.ingest_point_client import IngestPointClient


urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class AccessKeyClient(base_client.BaseClient):
    def __init__(self, north_host, insecure=False):
        self.north_host = north_host
        self.insecure = insecure

    def get_endpoint(self, ingest_point):
        endpoint = '{north_host}/ingest_points/{ingest_point}/access_keys'.format(
            north_host=self.north_host,
            ingest_point=ingest_point
        )
        return endpoint

    def create_access_key(self, auth_token, ingest_point, **kwargs):
        endpoint = self.get_endpoint(ingest_point)
        response = requests.post(
            endpoint,
            verify=(not self.insecure),
            headers={"X-Auth-Token": auth_token},
            params=self.get_auth_params(**kwargs)
        )
        response.raise_for_status()
        return json.loads(response.content)


def get_access_key(aaa_host, username, password, north_host, ingest_point, **kwargs):
    auth_client = AuthClient(aaa_host, insecure=kwargs['insecure'])
    auth_token = auth_client.get_token(username, password)

    ingest_point_client = IngestPointClient(north_host, insecure=kwargs['insecure'])
    ingest_point_info = ingest_point_client.get_ingest_point(auth_token, ingest_point, **kwargs)

    access_key_client = AccessKeyClient(north_host, insecure=kwargs['insecure'])
    access_key = access_key_client.create_access_key(auth_token, ingest_point, **kwargs)
    return ingest_point_info, access_key



from __future__ import unicode_literals

import os

import click

from cis_client import cli
from cis_client.commands import utils
from cis_client.lib.cis_gateway import http_upload_client
from cis_client.lib.cis_gateway import aspera_client
from cis_client.lib.cis_gateway import sftp_client
from cis_client import exception


@click.command('upload', context_settings=utils.CONTEXT_SETTINGS, help='Uploads content via HTTP/Aspera/SFTP.')
@click.option('--protocol', type=click.Choice(['http', 'aspera', 'sftp']), default='http',
              help='Protocol for loading. Can be http, aspera or sftp.')
@click.option('--ingest-point', required=True, type=click.STRING,
              help='Ingest point to load content to.')
@click.option('--source-file-list', type=click.Path(resolve_path=True),
              help='Path to file that contains list of full source paths to content separated by new line symbol. '
                   'Files from the list will be uploaded.')
@click.option('--source-file', type=click.STRING,
              help='Comma separated list of full source paths to content that will be uploaded. '
                   'Can contain path to directory with files if upload protocol is aspera.')
@click.option('--skip-errors', type=click.BOOL, default=False,
              help='Continue upload other files if some file upload was failed.')
@click.option('--destination-path', type=click.STRING,
              help='Destination path. '
                   'Can be directory like dir1/dir2/. If value ends with / destination '
                   'file path will be composed from this value + source filename. '
                   'If this value doesn\'t end with / so destination path will be the same.'
                   'In case when full destination path is specified input path list can contain only one input file.')
@utils.add_host_options
@utils.add_auth_options
@utils.add_credentials_options
@cli.pass_context
@utils.handle_exceptions
@utils.check_cis_version
def cli(ctx, **kwargs):
    paths = utils.get_source_file_list_from_kwargs(**kwargs)
    aaa_host = kwargs.pop('aaa_host')
    username = kwargs.pop('username')
    password = kwargs.pop('password')
    north_host = kwargs.pop('north_host')
    ingest_point = kwargs.pop('ingest_point')
    destination_path = kwargs.pop('destination_path')
    skip_errors = kwargs.pop('skip_errors')
    if destination_path and len(destination_path.split(',')) > 1:
        raise exception.OptionException(
            "Option --destination-path can contain only one path: "
            "directory with '/' suffix or full destination path for single source file.")
    if (destination_path and not destination_path.endswith('/') and
            len(paths) != 1):
        raise exception.OptionException(
            "Maybe you wanted to specify --destination-path={}/. "
            "Otherwise look into help page.".format(destination_path))

    for path in paths:
        if destination_path:
            if destination_path.endswith('/'):
                full_dst_path = ''.join([destination_path, os.path.basename(path)])
            else:
                full_dst_path = destination_path
        else:
            full_dst_path = os.path.basename(path)
        print("Uploading {} ...".format(path).encode('utf-8'))
        try:
            if kwargs['protocol'] == 'aspera':
                aspera_client.aspera_upload(
                    aaa_host, username, password,
                    north_host, ingest_point, path, destination_path=full_dst_path, **kwargs)
            else:
                total_size = os.path.getsize(path)
                with utils.ProgressBar(max_value=total_size) as progress_bar:
                    if kwargs['protocol'] == 'http':
                        response = http_upload_client.http_upload(
                            aaa_host, username, password, north_host,
                            ingest_point, path, destination_path=full_dst_path,
                            progress_callback=lambda transfered: progress_bar.update(transfered),
                            **kwargs)
                    elif kwargs['protocol'] == 'sftp':
                        response = sftp_client.sftp_upload(
                            aaa_host, username, password, north_host,
                            ingest_point, path, destination_path=full_dst_path,
                            progress_callback=lambda transfered, whole: progress_bar.update(transfered),
                            **kwargs)
        except Exception as e:
            if skip_errors:
                print("Error: {}".format(e).encode('utf-8'))
            else:
                raise
